#ifndef CHULL_H
#define CHULL_H

#include "pointset.h"
#include <QThread>
#include <QMutex>
#include <QSemaphore>

// constructs a convex hull.  it can do this as its own independent thread
class CHull : public QThread
{
    Q_OBJECT

    PointSet *_set;
    Point **_A;
    Point **_B;
    bool _abort;
    bool _restart;
    bool _running;
    QSemaphore _pause_count;

public:
    CHull(PointSet *set);
    ~CHull();

    void run();

    void next();
    void abort();
    void restart();

signals:
    void initialHull(bool bottom, int level, const Point *u, const Point *v);
    void nextEvent(double t, const Point *p, const Point *q, const Point *r);
    void updateHull(const Point *u, const Point *v);
    void doneLevel(bool bottom, int level); // level 0 is the root level, so 0 would mean the entire hull is constructed
    void done(bool aborted);

protected:
    static bool hasnil(const Point *p, const Point *q, const Point *r);
    // return > 1 if p-q-r is a clockwise/left turn, and < 1 if it is a ccw/right turn
    static double turn(const Point *p, const Point *q, const Point *r);
    // the time when p-q-r will change between a left/right turn (ie. become colinear)
    static double time(const Point *p, const Point *q, const Point *r);

    void hull(bool bottom, Point *list, int n, Point **A, Point **B, int level);
    void pauseit();
};

#endif // CHULL_H
