#ifndef PROJECTIONDISPLAY_H
#define PROJECTIONDISPLAY_H

#include "DGLWidget.h"
#include "pointset.h"

#include <QMouseEvent>

class ProjectionDisplay : public DGLWidget
{
    Q_OBJECT

public:
    static const int DEFAULT_ANIM_SPEED_MS = 1000; // ms

private:
    static const float POINT_SIZE = 3.0f;
    static const double HILIGHT_POINT_TIME = 500.0; //ms
    static const double ZOOM_TIME = 500.0; //ms

    PointSet *_set;
    double _cur_t;  // the time being displayed
    double _dest_t; // the time we are aiming to be displayed
    double _time_per_ms; // the amount of time to animate per ms

    double _hilite_cur;
    double _hilite_end;

    double _zoom_left;
    double _zoom_right;
    double _zoom_top;
    double _zoom_bottom;

    int _anim_speed_ms;  // the number of ms to take to show a whole animation

    const Point *_bridgel; // the current bridget vertices between the left and right hulls being merged
    const Point *_bridger;
    const Point *_hilite;

    enum {
        STATE_IDLE,
        STATE_HIGHLIGHTING,
        STATE_ZOOM_IN,
        STATE_ZOOM_OUT,
        STATE_TRANSLATING
    } _state;

    int _text_height;
    bool _auto_zoom;

    int _mousey;
    bool _zoom;
    bool _move;

public:
    ProjectionDisplay(QWidget *parent);
    ~ProjectionDisplay();

    void setPointSet(PointSet *p);
    void reset(); // resets the time to -infinity and just display the points
    void highlightPoint(const Point *event);
    void showTime(double t);
    void showHull(const Point *bridgel, const Point *bridger);

    void setAnimationSpeed(int ms); // one computational step is displayed in this many milliseconds
    int getAnimationSpeed() const { return _anim_speed_ms; }

    bool getAutoZoom() const { return _auto_zoom; }
    void setAutoZoom(bool z) { _auto_zoom = z; }

signals:
    void showingTime(double t);

protected:
    void initializeGL();
    void resizeGL(int w, int h);
    void paintGL();
    void paintGrid();
    void animate(int ms);
    void zoom();

    void mousePressEvent(QMouseEvent *e);
    void mouseReleaseEvent(QMouseEvent *e);
    void mouseMoveEvent(QMouseEvent *e);
};

#endif // PROJECTIONDISPLAY_H
